import { world, system } from "@minecraft/server";

export class TrapdoorAutoComponent {

    async onStepOn(event) {
        const block = event.block;
        const dimension = event.dimension;
        const x = block.x;
        const y = block.y;
        const z = block.z;

        try {
            // Obtener la permutación actual del bloque
            const permutation = block.permutation;
            const currentRotation = permutation.getState("minecraft:cardinal_direction");
            const currentState = permutation.getState("custom:geometry");
            
            // Determinar el nuevo estado basado en el estado actual
            const newState = currentState === 'abierto' ? 'cerrado' : 'abierto';

            // Crear una nueva permutación con el nuevo estado y la dirección actual
            const newPermutation = permutation.withState("custom:geometry", newState).withState("minecraft:cardinal_direction", currentRotation);
            
            // Aplicar la nueva permutación al bloque
            block.setPermutation(newPermutation);

            // Ejecutar comando para reproducir el sonido correspondiente
            //TODO: Esto hacerlo en condiciones que se escuche en un radio por todas las personas, no solo el jugador
            const soundCommand = newState === 'abierto'
                ? `playsound random.door_close @p ${x} ${y} ${z}`
                : `playsound random.door_open @p ${x} ${y} ${z}`;
            
            await dimension.runCommandAsync(soundCommand);

           
        } catch (error) {
            world.sendMessage(`Error: ${error}`);
        }
    }

    
    async onPlayerInteract(event) {
        const block = event.block;
        const player = event.player;
        const dimension = player.dimension;
        const x = block.x;
        const y = block.y;
        const z = block.z;

      
        try {
            // Obtener la permutación actual del bloque
            const permutation = block.permutation;
            const currentRotation = permutation.getState("minecraft:cardinal_direction");
            const currentState = permutation.getState("custom:geometry");
            
            // Determinar el nuevo estado basado en el estado actual
            const newState = currentState === 'abierto' ? 'cerrado' : 'abierto';

            // Crear una nueva permutación con el nuevo estado y la dirección actual
            const newPermutation = permutation.withState("custom:geometry", newState).withState("minecraft:cardinal_direction", currentRotation);
            
            // Aplicar la nueva permutación al bloque
            block.setPermutation(newPermutation);

            // Ejecutar comando para reproducir el sonido correspondiente
            //TODO: Esto hacerlo en condiciones que se escuche en un radio por todas las personas, no solo el jugador
            const soundCommand = newState === 'abierto'
                ? `playsound random.door_close @p ${x} ${y} ${z}`
                : `playsound random.door_open @p ${x} ${y} ${z}`;
            
            await dimension.runCommandAsync(soundCommand);

           
        } catch (error) {
            world.sendMessage(`Error: ${error}`);
        }
    }
}

export default TrapdoorAutoComponent;
